# -*- coding: utf-8 -*-
"""
Created on Fri Mar 21 15:32:12 2025

@author: mstep
"""
from PIL import Image

def set_image_margins(image, cutoff, margin):
    '''Size the image so it has margins of the indicated size.'''
    # Convert to RGB mode.
    image = image.convert('RGB')

    # Load the pixel data.
    pixels = image.load()

    # Find the non-white pixel bounds.
    xmax = 0
    xmin = image.width
    ymax = 0
    ymin = image.height
    for y in range(image.height):
        for x in range(image.width):
            r, g, b = pixels[x, y]
            if r + g + b < cutoff:
                if x < xmin: xmin = x
                if x > xmax: xmax = x
                if y < ymin: ymin = y
                if y > ymax: ymax = y

    # Crop to include just the bounding area.
    image = image.crop((xmin, ymin, xmax + 1, ymax + 1))

    # Make the new image.
    wid = 2 * margin + image.width
    hgt = 2 * margin + image.height
    new_image = Image.new('RGB', (wid, hgt), 'white')

    # Paste the cropped image onto the new image.
    new_image.paste(image, (margin, margin))

    # Return the result.
    return new_image

#%%

from PIL import Image

def resize_image(image, wid, hgt):
    '''Resize the image without distorting it.'''
    # Calculate the image's new size.
    image_wid = image.width
    image_hgt = image.height
    if wid / hgt > image_wid / image_hgt:
        # The image is too wide and short. Make it skinnier.
        wid = int(image_wid / image_hgt * hgt)
    else:
        # The image is too tall and thin. Make it shorter.
        hgt = int(wid / (image_wid / image_hgt))

    # Resize the image.
    return image.resize((wid, hgt))

#%%
import tkinter as tk
from PIL import Image, ImageTk
from tkinter import messagebox
from tkinter.filedialog import askopenfilename, asksaveasfilename

# Layout constants.
MARGIN = 5
WINDOW_WID = 400
WINDOW_HGT = 300

class SetImageMarginsApp:
    # Create and manage the tkinter interface.
    def __init__(self):
        # Object properties.
        self.raw_image = None

        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('pil_set_image_margins')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry(f'{WINDOW_WID}x{WINDOW_HGT}')

        # Build the menus.
        self.build_menus()

        # Build the rest of the UI here...
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, expand=False, fill=tk.X)

        label = tk.Label(frame, text='Margin:')
        label.pack(side=tk.LEFT, padx=(MARGIN, 2), pady=MARGIN)

        self.margin_var = tk.StringVar(value='10')
        self.margin_var.trace_add('write', self.margin_changed)
        entry = tk.Entry(frame, textvariable=self.margin_var, width=5,
            justify=tk.RIGHT)
        entry.pack(side=tk.LEFT, padx=(2, 2), pady=MARGIN)

        label = tk.Label(frame, text='Cutoff:')
        label.pack(side=tk.LEFT, padx=(20, 2), pady=MARGIN)

        self.cutoff_var = tk.StringVar(value='765')
        self.cutoff_var.trace_add('write', self.cutoff_changed)
        entry = tk.Entry(frame, textvariable=self.cutoff_var, width=5,
            justify=tk.RIGHT)
        entry.pack(side=tk.LEFT, padx=(2, 2), pady=MARGIN)

        self.canvas = tk.Canvas(self.window, bg='lightgray',
            borderwidth=0, highlightthickness=0)
        self.canvas.pack(side=tk.TOP, expand=True, fill=tk.BOTH,
            padx=MARGIN, pady=(0, MARGIN))

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def margin_changed(self, *args):
        self.show_image()

    def cutoff_changed(self, *args):
        self.show_image()

    def build_menus(self):
        '''Build the menu bar.'''
        menu_bar = tk.Menu(self.window)
        self.window.config(menu=menu_bar)
        self.build_file_menu(menu_bar)

    def build_file_menu(self, menu_bar):
        '''Build the File menu.'''
        file_menu = tk.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=file_menu)

        # Open
        self.open_photo_image = ImageTk.PhotoImage(file='open.png')
        file_menu.add_command(label='Open...', accelerator='Ctrl+O',
            compound='left', command=self.mnu_open,
            image=self.open_photo_image)
        self.window.bind('<Control-o>', lambda event: self.mnu_open())

        # Save As
        self.save_as_photo_image = ImageTk.PhotoImage(file='save_as.png')
        file_menu.add_command(label='Save As...', accelerator='Ctrl+S',
            compound='left', command=self.mnu_save_as,
            image=self.save_as_photo_image)
        self.window.bind('<Control-s>', lambda event: self.mnu_save_as())

        file_menu.add_separator()

        # Exit
        self.exit_photo_image = ImageTk.PhotoImage(file='exit.png')
        file_menu.add_command(label='Exit', compound='left',
            command=self.mnu_exit, image=self.exit_photo_image)

    def mnu_open(self):
        '''Open a file.'''
        filetypes = [
            ('Image Files', '*.png *.jpg *.jpeg *.bmp *.gif'),
            ('All Files', '*,*')]
        filename = askopenfilename(filetypes=filetypes)
        if len(filename) > 0:
            self.raw_image = Image.open(filename)
            self.show_image()

    def show_image(self):
        '''Display the image with the indicated margin.'''
        if self.raw_image is None: return

        # Get the margin and cutoff values.
        try:
            margin = int(self.margin_var.get())
        except:
            margin = 0

        try:
            cutoff = int(self.cutoff_var.get())
        except:
            cutoff = 3 * 255 # White.

        # Make sure the margin is no more than 1,000.
        if margin > 1000:
            messagebox.showerror('Margin Error',
                'The margin should be at most 1,000.')
            return

        # Set the image margins.
        self.margined_image = \
            set_image_margins(self.raw_image, cutoff, margin)

        # Size the image to fit.
        canvas_wid = self.canvas.winfo_width()
        canvas_hgt = self.canvas.winfo_height()
        self.resized_image = resize_image(self.margined_image,
            canvas_wid, canvas_hgt)

        # Display the image.
        self.photo_image = ImageTk.PhotoImage(self.resized_image)
        cx = canvas_wid / 2
        cy = canvas_hgt / 2
        self.canvas.delete(tk.ALL)
        self.canvas.create_image(cx, cy, anchor=tk.CENTER,
            image=self.photo_image)

    def mnu_save_as(self):
        '''Save the margined image.'''
        filetypes = [
            ('Image Files', '*.png *.jpg *.jpeg *.bmp *.gif'),
            ('All Files', '*,*')]
        filename = asksaveasfilename(filetypes=filetypes)
        if len(filename) > 0:
            self.margined_image.save(filename)

    def mnu_exit(self):
        self.kill_callback()

    def kill_callback(self):
        self.window.destroy()

#%%

SetImageMarginsApp()
print('Done')
